@@ -297,7 +297,7 @@ class S_statistic(Base_statistic):
         by_group = np.nanstd(data, ddof=1, axis=1)
         return GroupMeans(by_group, sum(sizes * by_group) / sum(sizes))
 
-    def sd(self, data, std_dev=None, sizes=None):
+    def sd(self, data, *, std_dev=None, sizes=None):
         if isinstance(std_dev, numbers.Number):
             return std_dev
 
@@ -431,7 +431,7 @@ class NP_statistic(Base_statistic):
             lcl = np.array([stats.binom(size, p).ppf(pbar) for size in sizes])
             ucl = np.array([stats.binom(size, p).isf(pbar) for size in sizes])
         lcl[lcl < 0] = 0
-        ucl[ucl > sizes] = sizes
+        ucl[ucl > sizes] = sizes[ucl > sizes]
         return pd.DataFrame({'LCL': lcl, 'UCL': ucl})
 
 # # c Chart
